__version__ = "0.4.9"
__git_commiter_name__ = "Arnon Yaari"
__git_commiter_email__ = "arnony@infinidat.com"
__git_branch__ = u'(HEAD detached at v0.4.9)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '401a5497ca6fadf8873c3884ee9d20f3f61199b7'
__git_head_subject__ = u'Finished Release v0.4.9'
__git_head_message__ = u''
__git_dirty_diff__ = u''
__git_commit_date__ = '2018-12-20 18:04:17'
